#include <math.h>

#include "SDL_prim.h"


/*
 * This is incorrect, as it can cause unintended transparency effects.  The
 * application should probably recognize that anti-aliased lines appear
 * brighter and reduce the intensity of the colors itself.
 *
 * // when anti-aliasing, the slightly fatter line appears brighter
 * // than the aliased equivalent, so a reduction in the alpha
 * // value makes it appear about the same, just smoother
 */
/* #define SDL_ALPHA_REDUCER	(200.0 / 255.0) */


/*
 * simple pixel drawing function.
 *
 * tests to be sure pixel is within bounds of the surface, so you
 * can draw off the screen to your hearts content
 *
 * it's probably not good practice to be drawing off the screen though...
 */
inline
void SDL_putPixel(SDL_Surface *surf, int x, int y, Uint32 clr)
{
	int Bpp = surf->format->BytesPerPixel;
	Uint8 *p;

	if (! (x < 0 || x >= surf->w || y < 0 || y >= surf->h) )
	{
		p = (Uint8*)surf->pixels + y * surf->pitch + x * Bpp;

		switch(Bpp) {
		case 1:
		  	*p = clr;
			break;

		case 2:
		  	*(Uint16*)p = clr;
			break;

		case 3:
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN) {
				p[0] = (clr >> 16) & 0xff;
				p[1] = (clr >> 8) & 0xff;
				p[2] = clr & 0xff;
			} else {
				p[0] = clr & 0xff;
				p[1] = (clr >> 8) & 0xff;
				p[2] = (clr >> 16) & 0xff;
			}
			break;

		case 4:
			*(Uint32*)p = clr;
			break;
		}
	}
}


/*
 * returns pointer to pixel at x, y
 */
inline
Uint8* SDL_getPixel(SDL_Surface *surf, int x, int y)
{
	int Bpp = surf->format->BytesPerPixel;

	if (! (x < 0 || x >= surf->w || y < 0 || y >= surf->h) )
		return (Uint8*)surf->pixels + y * surf->pitch + x * Bpp;
	else
		return 0;
}


/*
 * slow pixel blending function
 * used as a failover for 8 and 24 bpp
 * will be removed soon
 *
 * 8 bit doesn't work
 */
inline
void __slow_SDL_blendPixel( SDL_Surface *surf, int x, int y, Uint32 clr,
                            Uint8 alpha)
{
	Uint8 *p;
	Uint8 r, g, b, r2, g2, b2;

	if ( (p = SDL_getPixel(surf, x, y)) ) {
		SDL_GetRGB(clr, surf->format, &r, &g, &b);
		SDL_GetRGB(*p, surf->format, &r2, &g2, &b2);

		r = r2 + ( r - r2 ) * ( alpha / 255.0 );
		g = g2 + ( g - g2 ) * ( alpha / 255.0 );
		b = b2 + ( b - b2 ) * ( alpha / 255.0 );

		clr = SDL_MapRGB(surf->format, r, g, b);

		SDL_putPixel(surf, x, y, clr);
	}
}


/*
 * blends pixel onto surface according to alpha
 */
inline
void SDL_blendPixel( SDL_Surface *surf, int x, int y, Uint32 clr,
                     Uint8 alpha)
{
	Uint8 *p;
	Uint32 R, G, B;
	Uint32 Rmask = surf->format->Rmask;
	Uint32 Gmask = surf->format->Gmask;
	Uint32 Bmask = surf->format->Bmask;

	if ( (p = SDL_getPixel(surf, x, y)) ) {
		switch(surf->format->BytesPerPixel) {
		case 1:  /* 8 bpp */
			__slow_SDL_blendPixel(surf, x, y, clr, alpha);
			break;

		case 2:  /* 16 bpp */
			R = ( (*(Uint16*)p & Rmask) + ( ((clr & Rmask) - ((*(Uint16*)p) & Rmask)) * alpha >> 8) ) & Rmask;
			G = ( (*(Uint16*)p & Gmask) + ( ((clr & Gmask) - ((*(Uint16*)p) & Gmask)) * alpha >> 8) ) & Gmask;
			B = ( (*(Uint16*)p & Bmask) + ( ((clr & Bmask) - ((*(Uint16*)p) & Bmask)) * alpha >> 8) ) & Bmask;
			*(Uint16*)p = R | G | B;
			break;

		case 3:  /* 24 bpp */
			__slow_SDL_blendPixel(surf, x, y, clr, alpha);
			break;

		case 4:  /* 32 bpp */
			R = ( (*(Uint32*)p & Rmask) + ( ((clr & Rmask) - (*(Uint32*)p & Rmask)) * alpha >> 8) ) & Rmask;
			G = ( (*(Uint32*)p & Gmask) + ( ((clr & Gmask) - (*(Uint32*)p & Gmask)) * alpha >> 8) ) & Gmask;
			B = ( (*(Uint32*)p & Bmask) + ( ((clr & Bmask) - (*(Uint32*)p & Bmask)) * alpha >> 8) ) & Bmask;
			*(Uint32*)p = R | G | B;
			break;
		}
	}
}


/*
 * draw a line
 */
void SDL_drawLine( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                   Uint32 clr )
{
	int x, y, dx, dy, c;
	float slope;

	dx = x2 - x1;  /* change in x */
	dy = y2 - y1;  /* change in y */

	slope = (float)dy / dx;

	if (abs(dx) >= abs(dy)) {
		/* draw left/right - right/left */
		c = (x1 <= x2) ? 1 : -1;
		for (x = x1; x != x2 + c; x += c) {
			y = y1 + ( (x - x1) * slope + 0.5 );
			SDL_putPixel(surf, x, y, clr);
		}
	}
	else {
		/* draw top/bottom - bottom/top */
		c = (y1 <= y2) ? 1 : -1;
		for (y = y1; y != y2 + c; y += c) {
			x = x1 + ( (y - y1) / slope + 0.5 );
			SDL_putPixel(surf, x, y, clr);
		}
	}
}


/*
 * draw anti-aliased lines
 */
void SDL_drawLine_AA( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                      Uint32 clr )
{
	SDL_drawLine_Alpha_AA(surf, x1, y1, x2, y2, clr, 255);
}


/*
 * draw lines with alpha blending
 */
void SDL_drawLine_Alpha( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                         Uint32 clr, Uint8 alpha )
{
	int x, y, dx, dy, c;
	float slope;

	dx = x2 - x1;
	dy = y2 - y1;

	slope = (float)dy / dx;

	if (abs(dx) >= abs(dy)) {
		/* draw left/right - right/left */
		c = (x1 <= x2) ? 1 : -1;
		for (x = x1; x != x2 + c; x += c) {
			y = y1 + ( (x - x1) * slope + 0.5 );
			SDL_blendPixel(surf, x, y, clr, alpha);
		}
	}
	else {
		/* draw top/bottom - bottom/top */
		c = (y1 <= y2) ? 1 : -1;
		for (y = y1; y != y2 + c; y += c) {
			x = x1 + ( (y - y1) / slope + 0.5 );
			SDL_blendPixel(surf, x, y, clr, alpha);
		}
	}
}


/*
 * draw anti-aliased lines with alpha blending
 */
void SDL_drawLine_Alpha_AA( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                            Uint32 clr, Uint8 alpha )
{
	int x, y, dx, dy, c;
	float aa;
	float slope;

	dx = x2 - x1;
	dy = y2 - y1;

	/* alpha *= SDL_ALPHA_REDUCER; */

	slope = (float)dy / dx;

	if (abs(dx) >= abs(dy)) {
		/* draw left/right - right/left */
		c = (x1 <= x2) ? 1 : -1;
		for (x = x1; x != x2 + c; x += c) {
			/*
			 * aa is the fill percentage of the pixel adjacent
			 * the current one.  same for below
			 */
			aa = y1 + ( (x - x1) * slope + 0.5 );
			y = (int)aa;

			aa = ( aa - y ) * alpha;

			if (alpha > 254)
				SDL_putPixel(surf, x, y, clr);
			else
				SDL_blendPixel(surf, x, y, clr, alpha);
			SDL_blendPixel(surf, x, y + 1, clr, (int)aa);
			SDL_blendPixel(surf, x, y - 1, clr, alpha - (int)aa);
		}
	}
	else {
		/* draw top/bottom - bottom/top */
		c = (y1 <= y2) ? 1 : -1;
		for (y = y1; y != y2 + c; y += c) {
			aa = x1 + ( (y - y1) / slope + 0.5 );
			x = (int)aa;

			aa = ( aa - x ) * alpha;

			if (alpha > 254)
				SDL_putPixel(surf, x, y, clr);
			else
				SDL_blendPixel(surf, x, y, clr, alpha);
			SDL_blendPixel(surf, x + 1, y, clr, (int)aa);
			SDL_blendPixel(surf, x - 1, y, clr, alpha - (int)aa);
		}
	}
}


/*
 * draw a circle
 */
void SDL_drawCircle(SDL_Surface *surf, int x1, int y1, int r, Uint32 clr)
{
	int x, y;

	for (x = 0; x <= r / 1.4; x++) {
		y = 0.5 + sqrt( (r * r) - (x * x) );
		SDL_putPixel(surf, x1 + x, y1 + y, clr);
		SDL_putPixel(surf, x1 + x, y1 + -1 * y, clr);
		SDL_putPixel(surf, x1 + -1 * x, y1 + -1 * y, clr);
		SDL_putPixel(surf, x1 + -1 * x, y1 + y, clr);

		/* rotated 90 deg */

		SDL_putPixel(surf, x1 + y, y1 + x, clr);
		SDL_putPixel(surf, x1 + y, y1 + -1 * x, clr);
		SDL_putPixel(surf, x1 + -1 * y, y1 + -1 * x, clr);
		SDL_putPixel(surf, x1 + -1 * y, y1 + x, clr);
	}
}


/*
 * draw anti-aliased circle
 */
void SDL_drawCircle_AA(SDL_Surface *surf, int x1, int y1, int r, Uint32 clr)
{
	SDL_drawCircle_Alpha_AA(surf, x1, y1, r, clr, 255);
}


/*
 * draw anti-aliased circle with alpha blending
 */
void SDL_drawCircle_Alpha_AA( SDL_Surface *surf, int x1, int y1, int r,
                              Uint32 clr, Uint8 alpha )
{
	int x, y;
	float aa;

	/* alpha *= SDL_ALPHA_REDUCER; */

	for (x = 0; x <= r; x++) {
		aa = sqrt( (r * r) - (x * x) );
		y = (int)aa;
		aa = (aa - y) * alpha;

		/*
		 * x and y values converge 1/8 around the circle.
		 * we're drawing around the circle 8 different ways,
		 * so we need to stop to prevent overlapping.  this is
		 * very tricky as different radius circles will be off
		 * by one.
		 *
		 * as is, this function currently misses anti-aliasing
		 * 4 pixels on some circles.
		 */

		if (x <= y) {
			SDL_blendPixel(surf, x1 + x, y1 + y, clr, alpha);
			SDL_blendPixel(surf, x1 + x, y1 + y + 1, clr, aa);

			SDL_blendPixel(surf, x1 + x, y1 - y, clr, alpha);
			SDL_blendPixel(surf, x1 + x, y1 - y - 1, clr, aa);
		}
		if (x < y) {
			SDL_blendPixel(surf, x1 + x, y1 + y - 1, clr, alpha - aa);
			SDL_blendPixel(surf, x1 + x, y1 - y + 1, clr, alpha - aa);
		}

		if (x > 0) {
			if (x <= y) {
				SDL_blendPixel(surf, x1 - x, y1 + y, clr, alpha);
				SDL_blendPixel(surf, x1 - x, y1 + y + 1, clr, aa);
				SDL_blendPixel(surf, x1 - x, y1 - y, clr, alpha);
				SDL_blendPixel(surf, x1 - x, y1 - y - 1, clr, aa);
			}

			if (x < y) {
				SDL_blendPixel(surf, x1 - x, y1 + y - 1, clr, alpha - aa);
				SDL_blendPixel(surf, x1 - x, y1 - y + 1, clr, alpha - aa);
			}
		}

		/* rotated 90deg */

		if (x < y) {
			SDL_blendPixel(surf, x1 + y, y1 + x, clr, alpha);
			SDL_blendPixel(surf, x1 + y + 1, y1 + x, clr, aa);
			SDL_blendPixel(surf, x1 + y - 1, y1 + x, clr, alpha - aa);

			SDL_blendPixel(surf, x1 - y, y1 + x, clr, alpha);
			SDL_blendPixel(surf, x1 - y - 1, y1 + x, clr, aa);
			SDL_blendPixel(surf, x1 - y + 1, y1 + x, clr, alpha - aa);

			if (x > 0) {
				SDL_blendPixel(surf, x1 + y, y1 - x, clr, alpha);
				SDL_blendPixel(surf, x1 + y + 1, y1 - x, clr, aa);
				SDL_blendPixel(surf, x1 + y - 1, y1 - x, clr, alpha - aa);

				SDL_blendPixel(surf, x1 - y, y1 - x, clr, alpha);
				SDL_blendPixel(surf, x1 - y - 1, y1 - x, clr, aa);
				SDL_blendPixel(surf, x1 - y + 1, y1 - x, clr, alpha - aa);
			}
		}

		if (x > y)
			break;
	}
}


/*
 * draw a filled circle
 */
void SDL_fillCircle(SDL_Surface *surf, int x1, int y1, int r, Uint32 clr)
{
	int x, y;

	SDL_putPixel(surf, x1, y1, clr);
	for (x = 0; x <= r / 1.4; x++) {
		for (y = 0.5 + sqrt( (r * r) - (x * x) ); y > 0; --y) {
			SDL_putPixel(surf, x1 + x, y1 + y, clr);
			SDL_putPixel(surf, x1 + x, y1 + -1 * y, clr);
			SDL_putPixel(surf, x1 + -1 * x, y1 + -1 * y, clr);
			SDL_putPixel(surf, x1 + -1 * x, y1 + y, clr);

			/* rotated 90 deg */

			SDL_putPixel(surf, x1 + y, y1 + x, clr);
			SDL_putPixel(surf, x1 + y, y1 + -1 * x, clr);
			SDL_putPixel(surf, x1 + -1 * y, y1 + -1 * x, clr);
			SDL_putPixel(surf, x1 + -1 * y, y1 + x, clr);
		}
	}
}



/*
 * draw anti-aliased filled circle
 */
void SDL_fillCircle_AA( SDL_Surface *surf, int x1, int y1, int r, Uint32 clr)
{
	SDL_drawCircle_Alpha_AA(surf, x1, y1, r, clr, 255);
	SDL_fillCircle(surf, x1, y1, r - 1, clr);
}


/*
 * draw anti-aliased filled circle with alpha blending
 */
void _SDL_fillCircle_Alpha_AA( SDL_Surface *surf, int x1, int y1, int r,
                              Uint32 clr, Uint8 alpha, int doaa )
{
	int x, y;
	float aa = 0, py;

	for (x = 0; x <= r; x++) {
		py = aa;
		aa = sqrt( (r * r) - (x * x) );
		y = (int)aa;
		aa = (aa - y) * alpha;

		/*
		 * this part is similar to the above draw_Alpha_AA,
		 * but we only need to anti-alias the outer rim
		 * of the circle because the rest will be filled
		 */
		if (x <= y && doaa) {
			SDL_blendPixel(surf, x1 + x, y1 + y + 1, clr, aa);
			SDL_blendPixel(surf, x1 - x, y1 + y + 1, clr, aa);
			SDL_blendPixel(surf, x1 + x, y1 - y - 1, clr, aa);
			SDL_blendPixel(surf, x1 - x, y1 - y - 1, clr, aa);

			/* rotated 90 deg */

			SDL_blendPixel(surf, x1 + y + 1, y1 + x, clr, aa);
			SDL_blendPixel(surf, x1 - y - 1, y1 + x, clr, aa);
			SDL_blendPixel(surf, x1 + y + 1, y1 - x, clr, aa);
			SDL_blendPixel(surf, x1 - y - 1, y1 - x, clr, aa);
		}

		/* fill the circle */
		for (; y >= 0; y--) {
			SDL_blendPixel(surf, x1 + x, y1 + y, clr, alpha);
			if (y > 0)
				SDL_blendPixel(surf, x1 + x, y1 - y, clr, alpha);
			if (x > 0) {
				if (y > 0)
					SDL_blendPixel(surf, x1 - x, y1 + y, clr, alpha);
				SDL_blendPixel(surf, x1 - x, y1 - y, clr, alpha);
			}
		}
	}
}


/*
 * draw a filled circle with alpha blending, no aa
 */
void SDL_fillCircle_Alpha( SDL_Surface *surf, int x1, int y1, int r,
                           Uint32 clr, Uint8 alpha )
{
	_SDL_fillCircle_Alpha_AA( surf, x1, y1, r, clr, alpha, 0 );
}


/*
 * draw a filled circle with alpha blending and anti-aliasing
 */
void SDL_fillCircle_Alpha_AA( SDL_Surface *surf, int x1, int y1, int r,
                              Uint32 clr, Uint8 alpha )
{
	_SDL_fillCircle_Alpha_AA( surf, x1, y1, r, clr, alpha, 1 );
}


/*
 * draws a triangle
 */
void SDL_drawTriangle( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                       int x3, int y3, Uint32 clr )
{
	SDL_drawLine(surf, x1, y1, x2, y2, clr);
	SDL_drawLine(surf, x2, y2, x3, y3, clr);
	SDL_drawLine(surf, x3, y3, x1, y1, clr);
}


/*
 * draws an anti-aliased triangle
 */
void SDL_drawTriangle_AA( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                          int x3, int y3, Uint32 clr )
{
	SDL_drawLine_AA(surf, x1, y1, x2, y2, clr);
	SDL_drawLine_AA(surf, x2, y2, x3, y3, clr);
	SDL_drawLine_AA(surf, x3, y3, x1, y1, clr);
}


/*
 * draws an alpha-blended triangle
 */
void SDL_drawTriangle_Alpha( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                             int x3, int y3, Uint32 clr, Uint8 alpha )
{
	SDL_drawLine_Alpha(surf, x1, y1, x2, y2, clr, alpha);
	SDL_drawLine_Alpha(surf, x2, y2, x3, y3, clr, alpha);
	SDL_drawLine_Alpha(surf, x3, y3, x1, y1, clr, alpha);
}


/*
 * draws an alpha-blended anti-aliased triangle
 */
void SDL_drawTriangle_Alpha_AA( SDL_Surface *surf, int x1, int y1, int x2,
                                int y2, int x3, int y3, Uint32 clr,
                                Uint8 alpha )
{
	SDL_drawLine_Alpha_AA(surf, x1, y1, x2, y2, clr, alpha);
	SDL_drawLine_Alpha_AA(surf, x2, y2, x3, y3, clr, alpha);
	SDL_drawLine_Alpha_AA(surf, x3, y3, x1, y1, clr, alpha);
}


/*
 * draws a filled triangle
 */
void SDL_fillTriangle( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                       int x3, int y3, Uint32 clr )
{
	short int skipone = 1;
	int x, y, c, dx, dy, miny, maxy;
	float slope1, slope2, slope3;

	/*
	 * sort points by X coordinate so we can split the drawing
	 * at a slope change
	 */
	while ( !(x1 <= x2 && x1 <= x3) ) {
		x = x1;
		y = y1;
		x1 = x2;
		y1 = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}
	while ( !(x2 <= x3) ) {
		x = x2;
		y = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}

	dx = x3 - x1;
	dy = y3 - y1;
	slope1 = (float)dy / dx;

	dx = x1 - x2;
	dy = y1 - y2;
	slope2 = (float)dy / dx;

	dx = x2 - x3;
	dy = y2 - y3;
	slope3 = (float)dy / dx;

	/*
	 * draw between first two x,y coordinates
	 */
	if (x1 != x2) {
		for (x = x1; x <= x2; x++) {
			miny = y1 + ( (x - x1) * slope1 + 0.5 );
			maxy = y2 + ( (x - x2) * slope2 + 0.5 );
			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_putPixel(surf, x, y, clr);
			}
		}
	}
	else
		skipone = 0;

	/*
	 * draw between second two x,y coordinates
	 */
	if (x2 != x3) {
		for (x = x2 + skipone; x <= x3; x++) {
			miny = y1 + ( (x - x1) * slope1 + 0.5 );
			maxy = y3 + ( (x - x3) * slope3 + 0.5 );
			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_putPixel(surf, x, y, clr);
			}
		}
	}
}


/*
 * draw an anti-aliased filled triangle
 */
void SDL_fillTriangle_AA( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                          int x3, int y3, Uint32 clr )
{
	SDL_drawTriangle_AA( surf, x1, y1, x2, y2, x3, y3, clr );
	SDL_fillTriangle( surf, x1, y1, x2, y2, x3, y3, clr );
}


/*
 * draws a filled alpha-blended triangle
 */
void SDL_fillTriangle_Alpha( SDL_Surface *surf, int x1, int y1, int x2, int y2,
                             int x3, int y3, Uint32 clr, Uint8 alpha )
{
	short int skipone = 1;
	int x, y, c, dx, dy, miny, maxy;
	float slope1, slope2, slope3;

	/*
	 * sort points by X coordinate so we can split the drawing
	 * at a slope change
	 */
	while ( !(x1 <= x2 && x1 <= x3) ) {
		x = x1;
		y = y1;
		x1 = x2;
		y1 = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}
	while ( !(x2 <= x3) ) {
		x = x2;
		y = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}

	dx = x3 - x1;
	dy = y3 - y1;
	slope1 = (float)dy / dx;

	dx = x1 - x2;
	dy = y1 - y2;
	slope2 = (float)dy / dx;

	dx = x2 - x3;
	dy = y2 - y3;
	slope3 = (float)dy / dx;

	/*
	 * draw between first two x,y coordinates
	 */
	if (x1 != x2) {
		for (x = x1; x <= x2; x++) {
			miny = y1 + ( (x - x1) * slope1 + 0.5 );
			maxy = y2 + ( (x - x2) * slope2 + 0.5 );
			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_blendPixel(surf, x, y, clr, alpha);
			}
		}
	}
	else
		skipone = 0;

	/*
	 * draw between second two x,y coordinates
	 */
	if (x2 != x3) {
		for (x = x2 + skipone; x <= x3; x++) {
			miny = y1 + ( (x - x1) * slope1 + 0.5 );
			maxy = y3 + ( (x - x3) * slope3 + 0.5 );
			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_blendPixel(surf, x, y, clr, alpha);
			}
		}
	}
}


/*
 * draws a filled anti-aliased alpha-blended triangle
 */
void SDL_fillTriangle_Alpha_AA( SDL_Surface *surf, int x1, int y1, int x2,
                                int y2, int x3, int y3, Uint32 clr,
                                Uint8 alpha )
{
	short int skipone = 1;
	int x, y, c, dx, dy, miny, maxy;
	float slope1, slope2, slope3;
	float minaa, maxaa;

	/*
	 * sort points by X coordinate so we can split the drawing
	 * at a slope change
	 */
	while ( !(x1 <= x2 && x1 <= x3) ) {
		x = x1;
		y = y1;
		x1 = x2;
		y1 = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}
	while ( !(x2 <= x3) ) {
		x = x2;
		y = y2;
		x2 = x3;
		y2 = y3;
		x3 = x;
		y3 = y;
	}

	dx = x3 - x1;
	dy = y3 - y1;
	slope1 = (float)dy / dx;

	dx = x1 - x2;
	dy = y1 - y2;
	slope2 = (float)dy / dx;

	dx = x2 - x3;
	dy = y2 - y3;
	slope3 = (float)dy / dx;

	/*
	 * draw between first two x,y coordinates
	 */
	if (x1 != x2) {
		for (x = x1; x <= x2; x++) {
			minaa = y1 + ( (x - x1) * slope1 );
			maxaa = y2 + ( (x - x2) * slope2 );

			miny = (int)minaa;
			maxy = (int)maxaa;

			minaa = (minaa - miny) * alpha;
			maxaa = (maxaa - maxy) * alpha;

			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_blendPixel(surf, x, y, clr, alpha);
			}
			if (c > 0) {
				SDL_blendPixel(surf, x, miny - c, clr, alpha - minaa);
				SDL_blendPixel(surf, x, maxy + c, clr, maxaa);
			}
			else {
				SDL_blendPixel(surf, x, miny - c, clr, minaa);
				SDL_blendPixel(surf, x, maxy + c, clr, alpha - maxaa);
			}
		}
	}
	else
		skipone = 0;

	/*
	 * draw between second two x,y coordinates
	 */
	if (x2 != x3) {
		for (x = x2 + skipone; x <= x3; x++) {
			minaa = y1 + ( (x - x1) * slope1 );
			maxaa = y3 + ( (x - x3) * slope3 );

			miny = (int)minaa;
			maxy = (int)maxaa;

			minaa = (minaa - miny) * alpha;
			maxaa = (maxaa - maxy) * alpha;

			c = (miny < maxy) ? 1 : -1;
			for (y = miny; y != maxy + c; y += c) {
				SDL_blendPixel(surf, x, y, clr, alpha);
			}
			if (c > 0) {
				SDL_blendPixel(surf, x, miny - c, clr, alpha - minaa);
				SDL_blendPixel(surf, x, maxy + c, clr, maxaa);
			}
			else {
				SDL_blendPixel(surf, x, miny - c, clr, minaa);
				SDL_blendPixel(surf, x, maxy + c, clr, alpha - maxaa);
			}
		}
	}
}
